---------------------------
-- lets simplify a bit..
---------------------------
local gf = Glance.Functions
local gv = Glance.Variables
local ga = Glance.Arrays
local gb = Glance.Buttons
local gm = Glance.Menus

---------------------------
-- send message
---------------------------
function gf.sendMSG(text)
	print("|cff0279ffGlance:|r "..tostring(text));
end


---------------------------
-- debug messages
---------------------------
function Glance.Debug(what, which, msg)
	if gv.Debug then
		message = "Debug: "..what..": "..which
		if msg ~= nil then
			message = message.." ("..msg..")"
		end
		gf.sendMSG(message)
	end
end


---------------------------
-- add button to array
---------------------------
function gf.AddButton(name,anchor)
	gv.button.order[anchor][#gv.button.order[anchor]+1] = name
	gb[name] = {
		["name"] = name,
		["text"] = nil,
		["anchor"] = anchor,
		["font"] = gv.fnt_file,
		["position"] = #gv.button.order[anchor],
		["enabled"] = nil,
		["events"] = nil,
		["update"] = false,
		["onload"] = false,
		["tooltip"] = false,
		["menu"] = false,
		["click"] = false,
		["timer1"] = false,
		["timer5"] = false,
		["timerTooltip"] = false,
		["button"] = nil,
		["texture"] ={
			["normal"] = nil,
			["highlight"] = nil,
		},
		["save"] = {
			["perCharacter"] = nil,
			["perAccount"] = nil,
			["allowProfile"] = false,
		},
	}
	gf[name] = {}
	Glance.Debug("function","addButton",name)
end


---------------------------
-- add timer to array
---------------------------
function gf.AddTimer(name,low,high,reset,button,func,var)
	if gb[name].enabled then
		Glance.Timers[name] = {low,high,reset,button,func,var}
	end
end


---------------------------
-- add command to array
---------------------------
function gf.AddCommand(button,name,desc,func)
	if gb[button].enabled then
		Glance.Commands[name] = {
			["desc"] = desc,
			["func"] = func,
		}
	end
end


---------------------------
-- show tooltip
---------------------------
function gf.showTooltip(which)
	if gb[which].enabled then
		Glance.Debug("function","showtooltip",which)
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(gb[which].button, "ANCHOR_NONE")
		GameTooltip:SetPoint("TOPLEFT", gb[which].button, "BOTTOMLEFT", 0, -5)
		gf[which].tooltip()
		GameTooltip:Show()
	end
end


---------------------------
-- tooltip text
---------------------------
function gf.Tooltip.Title(text,color)
	local RGB = ga.colors.RGB
	GameTooltip:SetText(text, RGB[color][1], RGB[color][2], RGB[color][3])
end
function gf.Tooltip.Line(text,color)
	local RGB = ga.colors.RGB
	GameTooltip:AddLine(text, RGB[color][1], RGB[color][2], RGB[color][3])
end
function gf.Tooltip.Wrap(text,color)
	local RGB = ga.colors.RGB
	GameTooltip:AddLine(text, RGB[color][1], RGB[color][2], RGB[color][3], 1)
end
function gf.Tooltip.Double(text1,text2,col1,col2)
	local RGB = ga.colors.RGB
	GameTooltip:AddDoubleLine(text1, text2, RGB[col1][1], RGB[col1][2], RGB[col1][3], RGB[col2][1], RGB[col2][2], RGB[col2][3])
end
function gf.Tooltip.Space(text,color)
	gf.Tooltip.Line(" ","GLD")
end


---------------------------
-- set menu info
---------------------------
function gf.setInfo(d,i,h,n,c,t,v,l,f,ic,koc) --disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func,icon
	wipe(ga.info)
	ga.info = {
		["disabled"]     = d,
		["isTitle"]      = i,
		["hasArrow"]     = h,
		["notCheckable"] = n,
		["checked"]      = c,
		["text"]         = t,
		["value"]        = v,
		["func"]         = f,
		["icon"]         = ic,
		["keepShownOnClick"] = koc,
	}
	UIDropDownMenu_AddButton(ga.info, l)
end


---------------------------
-- bypass wow time played
---------------------------
function ChatFrame_DisplayTimePlayed(theFrame, totalTime, levelTime)
	if gv.requestingTimePlayed then
		gv.requestingTimePlayed = false
		gv.gametime.initTotal = time() - totalTime
		gv.gametime.initLevel = time() - levelTime
	else
		gf.ChatFrame_DisplayTimePlayed(theFrame, totalTime, levelTime)
	end
end


---------------------------
-- format time played string
---------------------------
function gf.formatTime(s)
	--return formatted
	local ts = SecondsToTime(s, false, false, 4)
	ts, count = string.gsub(ts,"Days","d")
	ts, count = string.gsub(ts,"Day","d")
	ts, count = string.gsub(ts,"Hr","h")
	ts, count = string.gsub(ts,"Min","m")
	ts, count = string.gsub(ts,"Sec","s")
	local a,b,c,d,e,f,g,h = strsplit(" ", ts) --gsub won't remove the spaces for whatever reason
	local tsnew = ""
	if a ~= nil and b ~= nil then tsnew = tsnew.." "..a..b end
	if c ~= nil and d ~= nil then tsnew = tsnew.." "..c..d end
	if e ~= nil and f ~= nil then tsnew = tsnew.." "..e..f end
	if g ~= nil and g ~= nil then tsnew = tsnew.." "..g..h end
	return tsnew
end


---------------------------
-- remove by value
---------------------------
function gf.tableRemoveByVal(tab, val)
	for k,v in pairs(tab) do
		if(v==val) then
			table.remove(tab, k);
			return true;
		end
	end
	return false;
end

---------------------------
-- add by value
---------------------------
function gf.tableAddByVal(tab, val)
	for k,v in pairs(tab) do
		if(v==val) then
			return false;
		end
	end
	table.insert(tab,val)
	return true;
end

---------------------------
-- min, max, avg
---------------------------
function gf.stats(var,num)
	var.current = num
	var.count = var.count + 1
	if (var.count == 1) then
		var.min = var.current
		var.max = var.current
		var.avg = var.current
	else
		if (var.current < var.min) then
			var.min = var.current
		elseif (var.current > var.max) then
			var.max = var.current
		end
		var.avg = math.floor((var.avg * (var.count-1) + var.current) / var.count)
	end
end


---------------------------
-- return pct
---------------------------
function gf.getPCT(small,large)
	return math.floor(((small/large)*100))
end

---------------------------
-- return rounded decimals
---------------------------
function gf.roundDecimal(what, decimals)
	return math.floor(what*math.pow(10,decimals)+0.5) / math.pow(10,decimals)
end

---------------------------
-- multi line text matching
---------------------------
function gf.match(u,t,find)
	for i = 1, u do
		local text = string.lower(_G[t..i]:GetText())
		for word in string.gmatch(text, "%S+") do
			if strfind(string.lower(word), string.lower(find)) then return true end
		end
	end
	return false
end

---------------------------
-- text matching
---------------------------
function gf.matchSingle(text,find)
	for word in string.gmatch(text, "%S+") do
		if strfind(string.lower(word), string.lower(find)) then return true end
	end
	return false
end

---------------------------
-- try to send a message
---------------------------
function gf.addonQuery(module)
	Glance.Debug("function","addonQuery",module)
	local GetNumPartyMembers, _ = GetNumSubgroupMembers()
	if (GetNumPartyMembers ~= 0 or gv.Debug) and Glance_Local.Options.sendStats and gv.messageCheck then
		if gv.Debug then
			SendAddonMessage("Glance", module.."^query^0", "WHISPER", UnitName("player"))
		else
			SendAddonMessage("Glance", module.."^query^0", "PARTY")
		end
		gv.messageCheck = false
	end
end

---------------------------
-- try to receive a msg
---------------------------
function gf.addonResponse(prefix, message, channel, sender)
	if prefix == "Glance" then
		Glance.Debug("function","addonResponse",sender..", "..message)
		local GetNumPartyMembers, _ = GetNumSubgroupMembers()
		if ((GetNumPartyMembers ~= 0 and sender ~= UnitName("player")) or gv.Debug) and Glance_Local.Options.sendStats then
			local module,which,data = strsplit("^", message)
			if which == "query" then
				local msg = gf[module].Message(data,sender)
				if gv.Debug then
					SendAddonMessage("Glance",module.."^response^"..msg, "WHISPER", UnitName("player"))
				else
					SendAddonMessage("Glance",module.."^response^"..msg, "PARTY")
				end
			elseif which == "response" then
				if sender == UnitName("party1") or sender == UnitName("player") then party = "a" end
				if sender == UnitName("party2") then party = "b" end
				if sender == UnitName("party3") then party = "c" end
				if sender == UnitName("party4") then party = "d" end
				gv.party[party].name = sender
				gv.party[party][module] = data
				gf.showTooltip(module)
			end
		end
	end
end

