---------------------------
-- lets simplify a bit..
---------------------------
local gf = Glance.Functions
local gv = Glance.Variables
local ga = Glance.Arrays
local gb = Glance.Buttons

---------------------------
-- create the button
---------------------------
gf.AddButton("Memory","LEFT")
local btn = gb.Memory
btn.text              = "Memory"
btn.events            = {"PLAYER_LOGIN"}
btn.enabled           = true
btn.update            = true
btn.click             = true
btn.onload            = true
btn.tooltip           = true
btn.timer5            = true
btn.timerTooltip      = true
btn.menu              = true
btn.save.perAccount = { ["collectGarbage"] = true, ["oneGlance"] = true }
local spa = btn.save.perAccount

---------------------------
-- variables
---------------------------
gv.memory = 0
gv.addons = {}
gv.gaddons = {}

---------------------------
-- timers
---------------------------
gf.AddTimer("Memory",0,5,true,nil,nil,nil)

---------------------------
-- update
---------------------------
function gf.Memory.update()
	local btn  = gb.Memory
	if btn.enabled and gv.loaded then -- loaded keeps it from launching when defined	
		local total = 0
		UpdateAddOnMemoryUsage()
		for i=1, GetNumAddOns() do
			if IsAddOnLoaded(i) then
				local memory = GetAddOnMemoryUsage(i)
				total = total + memory
			end
		end
		gv.memory = total
		btn.button:SetFormattedText("Mem: "..gf.Memory.FormatMemory(total))
		btn.button:SetWidth(btn.button:GetTextWidth())
	end
end

---------------------------
-- tooltip
---------------------------
function gf.Memory.tooltip()
	Glance.Debug("function","tooltip","Memory")
	local func = gf.Memory
	local tooltip = gf.Tooltip
	gf.Memory.update()
	local k,v,i
	local resort = false
	local acp = false
	for i=1, #gv.addons do
		if Glance.Timers.Memory[1] <= 1 then
			local mem = GetAddOnMemoryUsage(gv.addons[i][1])
			if gv.addons[i][1] == "Glance" and spa.oneGlance then
				local total = 0
				for j=1, #gv.gaddons do
					local gmemory = GetAddOnMemoryUsage(gv.gaddons[j][1])
					total = total + gmemory
					--print(gv.gaddons[j][1]..": "..func.FormatMemory(gmemory).." Total: "..func.FormatMemory(total))
				end
				mem = total
			end
			if gv.addons[i][2] ~= mem then
				if gv.addons[i][2] > mem then
					gv.addons[i][4] = "down"
				elseif gv.addons[i][2] < mem then
					gv.addons[i][4] = "up"
				end
				gv.addons[i][3] = "GRN"
				gv.addons[i][2] = mem
				resort = true
			else 
				gv.addons[i][3] = "LBL"
				gv.addons[i][4] = "steady"
			end
		else
			gv.addons[i][3] = "LBL"
			gv.addons[i][4] = "steady"
		end
	end
	--if Glance.Timers.Memory[1] <= 1 then
	if resort then
		table.sort(gv.addons, function(a, b) return a[2] > b[2] end)
		resort = false
	end
	tooltip.Title("Memory Usage", "GLD")
	for k, v in pairs(gv.addons) do
		if v[2] > 1000 then
			tooltip.Double("|T".."Interface\\Addons\\Glance_Memory\\"..v[4]..":0|t "..format("%s", v[1]), func.FormatMemory(v[2]), v[3], "RED")
		elseif v[2] > 500 then
			tooltip.Double("|T".."Interface\\Addons\\Glance_Memory\\"..v[4]..":0|t "..format("%s", v[1]), func.FormatMemory(v[2]), v[3], "YEL")
		else
			tooltip.Double("|T".."Interface\\Addons\\Glance_Memory\\"..v[4]..":0|t "..format("%s", v[1]), func.FormatMemory(v[2]), v[3], "GRN")
		end
		if v[1] == "ACP" then
			acp = true
		end
	end
	tooltip.Space()
	tooltip.Double("Total Memory Usage: ", func.FormatMemory(gv.memory), "WHT","WHT")
	tooltip.Line(" ","WHT")
	tooltip.Line("(Left-Click to force garbage collection)","WHT")
	tooltip.Line("(Right-Click to change Options)","WHT")
	if acp then
		tooltip.Line("(Shift-Click to launch ACP)","WHT")
	end
end

---------------------------
-- click
---------------------------
function gf.Memory.click(self, button, down)
	Glance.Debug("function","click","Memory")
	if (button == "LeftButton") then 
		if IsShiftKeyDown() then
			ShowUIPanel(ACP_AddonList)
		else
			gf.Memory.collectGarbage()
		end
	end
end

---------------------------
-- onload
---------------------------
function gf.Memory.onload()
	wipe(gv.addons)
	local i = 0
	UpdateAddOnMemoryUsage()
	if spa.oneGlance then
		local total = 0
		for i=1, GetNumAddOns() do
			local name, title = GetAddOnInfo(i)
			if gf.matchSingle(name,"Glance") then
				local memory = GetAddOnMemoryUsage(i)
				total = total + memory
				table.insert(gv.gaddons, {name, memory,"LBL","steady"})
			end
		end
		table.insert(gv.addons, {GetAddOnInfo("Glance"), total,"LBL","steady"})
	end
	for i=1, GetNumAddOns() do
		if IsAddOnLoaded(i) then
			local memory = GetAddOnMemoryUsage(i)
			local name, title = GetAddOnInfo(i)
			if not gf.matchSingle(name,"Glance") then
				table.insert(gv.addons, {GetAddOnInfo(i), memory,"LBL","steady"})
			elseif not spa.oneGlance then
				table.insert(gv.addons, {GetAddOnInfo(i), memory,"LBL","steady"})
			end
		end
	end
	table.sort(gv.addons, function(a, b) return a[2] > b[2] end)
	if spa.collectGarbage then
		gf.Memory.collectGarbage() 
	end
end


---------------------------
-- menu
---------------------------
function gf.Memory.menu(level,UIDROPDOWNMENU_MENU_VALUE)
	Glance.Debug("function","menu","Memory")
	local HEX = ga.colors.HEX
	level = level or 1
	if (level == 1) then
		gf.setInfo(nil,true,false,true,nil,HEX.white.."Memory Options","spacer",level,nil)
		gf.setInfo(nil,nil,true,true,nil,HEX.gold.."Collect Garbage on Login","garbage",level,nil)
		gf.setInfo(nil,nil,true,true,nil,HEX.gold.."Show only one entry for Glance","one",level,nil)
	end
	if (level == 2) then
		if UIDROPDOWNMENU_MENU_VALUE == "garbage" then
			local checked, func
			if spa.collectGarbage then checked = true else checked = false end
			func = function() spa.collectGarbage=true; end;
			--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func
			gf.setInfo(nil,false,false,false,checked,"On","On",level,func)
			if spa.collectGarbage then checked = false else checked = true end
			func = function() spa.collectGarbage=false; end;
			gf.setInfo(nil,false,false,false,checked,"Off","Off",level,func)
		end
		if UIDROPDOWNMENU_MENU_VALUE == "one" then
			local checked, func
			if spa.oneGlance then checked = true else checked = false end
			func = function() spa.oneGlance=true; gf.Memory.onload() end;
			--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func
			gf.setInfo(nil,false,false,false,checked,"On","On",level,func)
			if spa.oneGlance then checked = false else checked = true end
			func = function() spa.oneGlance=false; gf.Memory.onload() end;
			gf.setInfo(nil,false,false,false,checked,"Off","Off",level,func)
		end
	end
end

---------------------------
-- format memory usage
---------------------------
function gf.Memory.FormatMemory(usage)
	local HEX = ga.colors.HEX
	if usage > 1000 then
		return format(HEX.red.."%.2f "..HEX.white.."mb", usage/1024)
	elseif usage > 500 then
		return format(HEX.yellow.."%.2f "..HEX.white.."kb", usage)
	elseif usage > 0 then
		return format(HEX.green.."%.2f "..HEX.white.."kb", usage)
	end
end

---------------------------
-- garbage collection
---------------------------
function gf.Memory.collectGarbage()
	local current = collectgarbage("count")
	collectgarbage("collect")
	UpdateAddOnMemoryUsage()
	local garbage = current - collectgarbage("count")
	gf.sendMSG("Garbage Collected. ("..gf.Memory.FormatMemory(garbage)..")");
end


---------------------------
-- load on demand
---------------------------
if gv.loaded then
	gf.Enable("Memory")
end